/*
This program has been developed by students from the bachelor Computer Science at Utrecht University within the Software Project course.
© Copyright Utrecht University (Department of Information and Computing Sciences)
*/

package produce

import (
	"context"
	"testing"

	"git.science.uu.nl/datastrophe/broker"
	"git.science.uu.nl/datastrophe/keyvaluestore"
	"github.com/stretchr/testify/assert"
)

/*
TestPublishCorrectMessage test to be sure the correct message gets published
	t: *testing.T, makes go recognise this as a test
*/
func TestPublishCorrectMessage(t *testing.T) {
	// Create a mock broker
	mockBroker := broker.NewMockDriver().(*broker.MockDriver)
	// Create mock key value store
	keyValueStore := keyvaluestore.NewMockDriver()
	// Create new service and start it
	service := NewService(mockBroker, keyValueStore)
	service.Start()

	// Create mock session and mock queue
	mockSession := "mock-session"
	mockQueue := "mock-queue"

	// Set the test-session sessionID queue to mock-queue in key value store
	keyValueStore.Set(context.Background(), mockSession, mockQueue)

	// Create headers containing a sessionID
	headers := make(map[string]interface{})
	headers["sessionID"] = mockSession

	// Assert that there have not been any messages sent yet
	assert.Empty(t, mockBroker.Messages)

	// Publish the message
	mockMessage := []byte("Test message")
	service.PublishMessage(&mockMessage, &mockSession)

	// Assert that there is now 1 message
	assert.Len(t, mockBroker.Messages[mockQueue], 1)

	// Assert that this message contains the mockMessage
	assert.Equal(t, mockBroker.Messages[mockQueue][0].Body, mockMessage)
}

/*
TestPublishMessageNoRouting tests publishing the message without setting routing in key value store
	t: *testing.T, makes go recognise this as a test
*/
func TestPublishMessageNoRouting(t *testing.T) {
	// Create a mock broker
	mockBroker := broker.NewMockDriver().(*broker.MockDriver)
	// Create mock key value store
	keyValueStore := keyvaluestore.NewMockDriver()
	// Create new service and start it
	service := NewService(mockBroker, keyValueStore)
	service.Start()

	// Create mock session and mock queue
	mockSession := "mock-session"

	// Create headers containing a sessionID
	headers := make(map[string]interface{})
	headers["sessionID"] = mockSession

	// Assert that there have not been any messages sent yet
	assert.Empty(t, mockBroker.Messages)

	// Publish the message
	mockMessage := []byte("Test message")
	service.PublishMessage(&mockMessage, &mockSession)

	// Assert that there are still 0 messages
	assert.Empty(t, mockBroker.Messages)
}
